﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Lobs.DataModel
{
  public static class DataProviderFactory
  {
    public static IDataProvider GetProvider(string providerName)
    {
      return (from info in _providerInfos where info.Name == providerName select info).First().Singleton;
    }

    public static void RegisterProvider(string providerName, Type provider, string parameter)
    {
      _providerInfos.Add(new ProviderInfo { Name = providerName, Type = provider, Parameter = parameter });
    }

    public static void RegisterProvider(string providerName, Func<IDataProvider> creator)
    {
      _providerInfos.Add(new ProviderInfo { Name = providerName, Creator = creator });
    }

    /// <summary>
    /// Deinitializes the factory. Used for unit testing only.
    /// </summary>
    public static void Deinitialize()
    {
      _providerInfos.Clear();
    }

    class ProviderInfo
    {
      public string Name { get; set; }

      public string Parameter { get; set; }

      public Type Type { get; set; }

      public Func<IDataProvider> Creator { get; set; }

      public IDataProvider Singleton
      {
        get
        {
          if (_singleton == null)
          {
            if (Creator != null)
              _singleton = Creator();
            else
              _singleton = (IDataProvider)Activator.CreateInstance(Type, Parameter);
          }
          return _singleton;
        }
      }
      IDataProvider _singleton;
    }

    static readonly List<ProviderInfo> _providerInfos = new List<ProviderInfo>();
  }
}
