﻿Public Class DataProviderFactory

  Shared Sub New()
    DataProviderFactory._providerInfos = New List(Of ProviderInfo)
  End Sub

  Public Shared Sub Deinitialize()
    DataProviderFactory._providerInfos.Clear()
  End Sub

  Public Shared Function GetProvider(ByVal providerName As String) As IDataProvider
    Return DataProviderFactory._providerInfos.Where(Function(info As ProviderInfo) info.Name = providerName).First().Singleton
  End Function

  Public Shared Sub RegisterProvider(ByVal providerName As String, ByVal creator As Func(Of IDataProvider))
    Dim pi As New ProviderInfo
    pi.Name = providerName
    pi.Creator = creator
    DataProviderFactory._providerInfos.Add(pi)
  End Sub

  Public Shared Sub RegisterProvider(ByVal providerName As String, ByVal provider As Type, ByVal parameter As String)
    Dim providerInfo As New ProviderInfo
    providerInfo.Name = providerName
    providerInfo.Type = provider
    providerInfo.Parameter = parameter
    DataProviderFactory._providerInfos.Add(providerInfo)
  End Sub

  Private Shared ReadOnly _providerInfos As List(Of ProviderInfo)

  Private Class ProviderInfo
    Public Sub New()
    End Sub

    Public Property Creator() As Func(Of IDataProvider)
      Get
        Return _Creator
      End Get
      Set(ByVal value As Func(Of IDataProvider))
        _Creator = value
      End Set
    End Property
    Private _Creator As Func(Of IDataProvider)

    Public Property Name() As String
      Get
        Return _Name
      End Get
      Set(ByVal value As String)
        _Name = value
      End Set
    End Property
    Private _Name As String

    Public Property Parameter() As String
      Get
        Return _Parameter
      End Get
      Set(ByVal value As String)
        _Parameter = value
      End Set
    End Property
    Private _Parameter As String

    Public ReadOnly Property Singleton() As IDataProvider
      Get
        If (Me._Singleton Is Nothing) Then
          If (Not Me.Creator Is Nothing) Then
            Me._Singleton = Me.Creator.Invoke
          Else
            Me._Singleton = DirectCast(Activator.CreateInstance(Me.Type, New Object() {Me.Parameter}), IDataProvider)
          End If
        End If
        Return Me._Singleton
      End Get
    End Property
    Private _Singleton As IDataProvider

    Public Property Type() As Type
      Get
        Return _Type
      End Get
      Set(ByVal value As Type)
        _Type = value
      End Set
    End Property
    Private _Type As Type
  End Class
End Class
