﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.OleDb;
using System.IO;
using System.Linq;
using System.Text;
using System.Data.DataSetExtensions;
using Lobs.DataModel;

namespace Lobs.DataModel.AccessDataProvider
{
  public class MSAccessProvider : IDataProvider
  {
    public MSAccessProvider(string parameter)
    {
      _mdbFilename = parameter;

      // I found that out anew again and again...
      if (Is64BitMode())
        throw new Exception("OLEDB.4 does not run under 64 bit operation systems. Set 'Platform target' to 'x86'.");
    }

    static bool Is64BitMode()
    {
      return System.Runtime.InteropServices.Marshal.SizeOf(typeof(IntPtr)) == 8;
    }

    public static MSAccessProvider CreateDataProvider(string parameter)
    {
      return new MSAccessProvider(parameter);
    }

    //public void Initialize()
    //{
    //  GetConnection();
    //}

    //public void Deinitialize()
    //{
    //  CloseConnection();
    //}

    /// <summary>
    /// Loads the categories from the data base.
    /// </summary>
    public IEnumerable<Category> LoadCategories()
    {
      const string sql = "Select * From Categories";

      var categories = new List<Category>();
      try
      {
        var adapter = new OleDbDataAdapter(new OleDbCommand(sql, GetConnection()));
        var dataset = new DataSet();

        adapter.Fill(dataset, "Categories");
        adapter.FillSchema(dataset.Tables[0], SchemaType.Mapped);

        var table = dataset.Tables["Categories"];

        foreach (DataRow row in table.Rows)
        {
          var category = new Category();

          category.ID = row.Field<int>("CategoryID").ToString();
          category.Name = row.Field<string>("CategoryName");
          category.Description = row.Field<string>("Description");
          category.Picture = row.Field<byte[]>("Picture");

          categories.Add(category);
        }
      }
      finally
      {
        // nothing to do
      }
      return categories;
    }

    /// <summary>
    /// Loads the shippers from the data base.
    /// </summary>
    public IEnumerable<Shipper> LoadShippers()
    {
      const string sql = "Select * From Shippers";

      var shippers = new List<Shipper>();
      try
      {
        var adapter = new OleDbDataAdapter(new OleDbCommand(sql, GetConnection()));
        var dataset = new DataSet();

        adapter.Fill(dataset, "Shippers");
        adapter.FillSchema(dataset.Tables[0], SchemaType.Mapped);

        var table = dataset.Tables["Shippers"];

        foreach (DataRow row in table.Rows)
        {
          var shipper = new Shipper();

          shipper.ID = row.Field<int>("ShipperID").ToString();
          shipper.Name = row.Field<string>("CompanyName");

          shippers.Add(shipper);
        }
      }
      finally
      {
        // nothing to do
      }
      return shippers;
    }

    /// <summary>
    /// Loads the employees from the data base.
    /// </summary>
    public IEnumerable<Employee> LoadEmployees()
    {
      const string sql = "Select * From Employees";

      var employees = new List<Employee>();
      try
      {
        var adapter = new OleDbDataAdapter(new OleDbCommand(sql, GetConnection()));
        var dataset = new DataSet();

        adapter.Fill(dataset, "Employees");
        adapter.FillSchema(dataset.Tables[0], SchemaType.Mapped);

        var table = dataset.Tables["Employees"];

        foreach (DataRow row in table.Rows)
        {
          var employee = new Employee();
          employee.ID = row.Field<int>("EmployeeID").ToString();
          employee.LastName = row.Field<string>("LastName");
          employee.FirstName = row.Field<string>("FirstName");
          employee.TitleOfCourtesy = row.Field<string>("TitleOfCourtesy");
          employee.Title = row.Field<string>("Title");
          //employee.Photo = row.Field<byte[]>("Photo");

          employees.Add(employee);
        }
      }
      finally
      {
        // nothing to do
      }
      return employees;
    }

    public byte[] LoadEmployeePhoto(string id)
    {
      string sql = String.Format("select Photo from Employees where EmployeeID={0}", id);
      try
      {
        var adapter = new OleDbDataAdapter(new OleDbCommand(sql, GetConnection()));
        var dataset = new DataSet();

        adapter.Fill(dataset, "Employees");
        adapter.FillSchema(dataset.Tables[0], SchemaType.Mapped);

        var table = dataset.Tables["Employees"];

        if (table.Rows.Count == 1)
          return BitmapConverter.Convert(table.Rows[0].Field<byte[]>("Photo"));
      }
      finally
      {
        // nothing to do
      }
      return null;
    }

    public void SaveEmployee(Employee employee)
    {
      throw new NotImplementedException();
    }

    public void DeleteEmployee(Employee employee)
    {
      throw new NotImplementedException();
    }

    public int SaveEmployees(IEnumerable<Employee> employees)
    {
      throw new NotImplementedException();
    }

    public IEnumerable<Product> LoadProducts()
    {
      const string sql = "Select * From Products";

      var employees = new List<Product>();
      try
      {
        OleDbDataAdapter adapter = new OleDbDataAdapter(new OleDbCommand(sql, GetConnection()));

        DataSet dataset = new DataSet();
        adapter.Fill(dataset, "Products");
        adapter.FillSchema(dataset.Tables[0], SchemaType.Mapped);

        DataTable table = dataset.Tables["Products"];

        foreach (DataRow row in table.Rows)
        {
          var product = new Product();
          product.ID = row.Field<int>("ProductID").ToString();
          product.Name = row.Field<string>("ProductName");
          product.CategoryID = row.Field<int>("CategoryID").ToString();
          product.UnitPrice = row.Field<decimal>("UnitPrice");

          employees.Add(product);
        }
      }
      finally
      {
        // nothing to do
      }
      return employees;
    }

    public void SaveProduct(Product product)
    {
      throw new NotImplementedException();
    }


    public void Initialize(string connectString)
    {
      _connectString = connectString;
    }


    private OleDbConnection GetConnection()
    {
      //_mdbFilename;
      if (_connection == null)
      {
        AppDomain ad = AppDomain.CurrentDomain;
        string baseDirectory = ad.BaseDirectory;

        string file = Path.Combine(baseDirectory, @"..\..\..\Database\\Nwind.mdb"); // passt bei Aufruf durch WebService
        if (!File.Exists(file))
        {
          file = Path.Combine(baseDirectory, @"..\..\..\..\..\Database\Nwind.mdb"); // passt bei Aufruf durch Gkvm.Server.TestModels
          if (!File.Exists(file))
          {
            throw new Exception();

            //  file = Path.Combine(baseDirectory, "..\\..\\..\\Database\\GKVM.mdb"); // ???
            //  if (!File.Exists(file))
            //  {
            //    file = Path.Combine(baseDirectory, @"F:\@thho\PKV-GKVM-1.00\dev\Database\GKVM.mdb"); // Hack: Fallback
            //    if (!File.Exists(file))
            //    {
            //      file = Path.Combine(baseDirectory, @"D:\WebApps\projects,pkv,gkvm,recherche\Database\GKVM.mdb"); // Hack: Fallback
            //    }
            //  }
          }
        }

        string connect = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + file;
        _connection = new OleDbConnection(connect);
      }

      if (_connection != null && _connection.State == ConnectionState.Closed)
        _connection.Open();

      return _connection;
    }

    string _mdbFilename;

    private void CloseConnection()
    {
      if (_connection != null && _connection.State != ConnectionState.Closed)
        _connection.Close();

      _connection = null;
    }

    public void Dispose()
    {
      CloseConnection();
    }

    private string _connectString;
    private OleDbConnection _connection;
  }
}