﻿Imports System.Data.OleDb
Imports System.IO
Imports System.Runtime.InteropServices

Public Class MSAccessProvider
  Implements IDataProvider
  ' Methods
  Public Sub New(ByVal parameter As String)
    Me._mdbFilename = parameter
    If MSAccessProvider.Is64BitMode Then
      Throw New Exception("OLEDB.4 does not run under 64 bit operation systems. Set 'Platform target' to 'x86'.")
    End If
  End Sub

  Private Sub CloseConnection()
    If ((Not Me._connection Is Nothing) AndAlso (Me._connection.State <> ConnectionState.Closed)) Then
      Me._connection.Close()
    End If
    Me._connection = Nothing
  End Sub

  Public Shared Function CreateDataProvider(ByVal parameter As String) As MSAccessProvider
    Return New MSAccessProvider(parameter)
  End Function

  Public Sub DeleteEmployee(ByVal employee As Employee) Implements IDataProvider.DeleteEmployee
    Throw New NotImplementedException
  End Sub

  Public Sub Dispose()
    Me.CloseConnection()
  End Sub

  Private Function GetConnection() As OleDbConnection
    If (Me._connection Is Nothing) Then
      Dim baseDirectory As String = AppDomain.CurrentDomain.BaseDirectory
      Dim file As String = Path.Combine(baseDirectory, "..\..\..\Database\\Nwind.mdb")
      If Not IO.File.Exists(file) Then
        file = Path.Combine(baseDirectory, "..\..\..\..\..\Database\Nwind.mdb")
        If Not IO.File.Exists(file) Then
          Throw New Exception
        End If
      End If
      Dim connect As String = ("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & file)
      Me._connection = New OleDbConnection(connect)
    End If
    If ((Not Me._connection Is Nothing) AndAlso (Me._connection.State = ConnectionState.Closed)) Then
      Me._connection.Open()
    End If
    Return Me._connection
  End Function

  Public Sub Initialize(ByVal connectString As String)
    Me._connectString = connectString
  End Sub

  Private Shared Function Is64BitMode() As Boolean
    Return (Marshal.SizeOf(GetType(IntPtr)) = 8)
  End Function
  ' Function LoadCategories() As System.Collections.Generic.IEnumerable(Of Category)
  Public Function LoadCategories() As IEnumerable(Of Category) Implements IDataProvider.LoadCategories
    Dim categories As New List(Of Category)
    Try
      Dim adapter As New OleDbDataAdapter(New OleDbCommand("Select * From Categories", Me.GetConnection))
      Dim dataset As New DataSet
      adapter.Fill(dataset, "Categories")
      adapter.FillSchema(dataset.Tables.Item(0), SchemaType.Mapped)
      Dim table As DataTable = dataset.Tables.Item("Categories")
      Dim row As DataRow
      For Each row In table.Rows
        Dim category As New Category
        category.ID = row.Field(Of Integer)("CategoryID").ToString
        category.Name = row.Field(Of String)("CategoryName")
        category.Description = row.Field(Of String)("Description")
        category.Picture = row.Field(Of Byte())("Picture")
        categories.Add(category)
      Next
    Catch
    End Try
    Return categories
  End Function

  Public Function LoadEmployeePhoto(ByVal id As String) As Byte() Implements IDataProvider.LoadEmployeePhoto
    Dim sql As String = String.Format("select Photo from Employees where EmployeeID={0}", id)
    Try
      Dim adapter As New OleDbDataAdapter(New OleDbCommand(sql, Me.GetConnection))
      Dim dataset As New DataSet
      adapter.Fill(dataset, "Employees")
      adapter.FillSchema(dataset.Tables.Item(0), SchemaType.Mapped)
      Dim table As DataTable = dataset.Tables.Item("Employees")
      If (table.Rows.Count = 1) Then
        Return BitmapConverter.Convert(table.Rows.Item(0).Field(Of Byte())("Photo"))
      End If
    Catch
    End Try
    Return Nothing
  End Function

  Public Function LoadEmployees() As IEnumerable(Of Employee) Implements IDataProvider.LoadEmployees
    Dim employees As New List(Of Employee)
    Try
      Dim adapter As New OleDbDataAdapter(New OleDbCommand("Select * From Employees", Me.GetConnection))
      Dim dataset As New DataSet
      adapter.Fill(dataset, "Employees")
      adapter.FillSchema(dataset.Tables.Item(0), SchemaType.Mapped)
      Dim table As DataTable = dataset.Tables.Item("Employees")
      Dim row As DataRow
      For Each row In table.Rows
        Dim employee As New Employee
        employee.ID = row.Field(Of Integer)("EmployeeID").ToString
        employee.LastName = row.Field(Of String)("LastName")
        employee.FirstName = row.Field(Of String)("FirstName")
        employee.TitleOfCourtesy = row.Field(Of String)("TitleOfCourtesy")
        employee.Title = row.Field(Of String)("Title")
        employees.Add(employee)
      Next
    Catch
    End Try
    Return employees
  End Function

  Public Function LoadProducts() As IEnumerable(Of Product) Implements IDataProvider.LoadProducts
    Dim employees As New List(Of Product)
    Try
      Dim adapter As New OleDbDataAdapter(New OleDbCommand("Select * From Products", Me.GetConnection))
      Dim dataset As New DataSet
      adapter.Fill(dataset, "Products")
      adapter.FillSchema(dataset.Tables.Item(0), SchemaType.Mapped)
      Dim table As DataTable = dataset.Tables.Item("Products")
      Dim row As DataRow
      For Each row In table.Rows
        Dim product As New Product
        product.ID = row.Field(Of Integer)("ProductID").ToString
        product.Name = row.Field(Of String)("ProductName")
        product.CategoryID = row.Field(Of Integer)("CategoryID").ToString
        product.UnitPrice = row.Field(Of Decimal)("UnitPrice")
        employees.Add(product)
      Next
    Catch
    End Try
    Return employees
  End Function

  Public Function LoadShippers() As IEnumerable(Of Shipper) Implements IDataProvider.LoadShippers
    Dim shippers As New List(Of Shipper)
    Try
      Dim adapter As New OleDbDataAdapter(New OleDbCommand("Select * From Shippers", Me.GetConnection))
      Dim dataset As New DataSet
      adapter.Fill(dataset, "Shippers")
      adapter.FillSchema(dataset.Tables.Item(0), SchemaType.Mapped)
      Dim table As DataTable = dataset.Tables.Item("Shippers")
      Dim row As DataRow
      For Each row In table.Rows
        Dim shipper As New Shipper
        shipper.ID = row.Field(Of Integer)("ShipperID").ToString
        shipper.Name = row.Field(Of String)("CompanyName")
        shippers.Add(shipper)
      Next
    Catch
    End Try
    Return shippers
  End Function

  Public Sub SaveEmployee(ByVal employee As Employee) Implements IDataProvider.SaveEmployee
    Throw New NotImplementedException
  End Sub

  Public Function SaveEmployees(ByVal employees As IEnumerable(Of Employee)) As Integer Implements IDataProvider.SaveEmployees
    Throw New NotImplementedException
  End Function

  Public Sub SaveProduct(ByVal product As Product) Implements IDataProvider.SaveProduct
    Throw New NotImplementedException
  End Sub

  ' Fields
  Private _connection As OleDbConnection
  Private _connectString As String
  Private _mdbFilename As String
End Class
