﻿using System;
using System.ComponentModel;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Lobs.ViewModel
{
  public class Employee : ViewModelBase
  {
    public string LastName
    {
      get { return _lastName; }
      set { if (_lastName != value) { _lastName = value; RaisePropertyChanged(() => LastName); RaisePropertyChanged(() => FullName); } }
    }

    private string _lastName;

    public string FirstName
    {
      get { return _firstName; }
      set { if (_firstName != value) { _firstName = value; RaisePropertyChanged(() => FirstName); RaisePropertyChanged(() => FullName); } }
    }

    private string _firstName;

    public string TitleOfCourtesy
    {
      get { return _titleOfCourtesy; }
      set { if (_titleOfCourtesy != value) { _titleOfCourtesy = value; RaisePropertyChanged(() => TitleOfCourtesy); RaisePropertyChanged(() => FullName); } }
    }

    private string _titleOfCourtesy;

    public string FullName
    {
      get { return String.Format("{0} {1} {2}", TitleOfCourtesy, FirstName, LastName); }
    }

    public string Title
    {
      get { return _title; }
      set { if (_title != value) { _title = value; RaisePropertyChanged(() => Title); } }
    }

    private string _title;

    public ImageSource Photo
    {
      get { return _photo; ; }
      set { if (_photo != value) { _photo = value; RaisePropertyChanged(() => Photo); } }
    }

    ImageSource _photo;

    public byte[] PhotoBytes
    {
      get { return _photoBytes; }
      set
      {
        _photoBytes = value;
#if !SILVERLIGHT
        var encoder = new JpegBitmapDecoder(new MemoryStream(value), BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
        Photo = encoder.Frames[0];
#else
        var bm = new BitmapImage();
        bm.SetSource(new MemoryStream(value));
        Photo = bm;
#endif

#if WPF_only
        // NorthWind is an old data base. WPF can read this old bitmaps, while Silverlight cannot.
        // This code is not in use anymore because we convert the 16 color Window 1.0 bitmap into a jpeg.
        var stream = new MemoryStream();
        const int offset = 78;
        stream.Write(value, offset, value.Length - offset);

        // Create a bitmap image
        var encoder = new BmpBitmapDecoder(stream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
        Photo = new Image { Source = encoder.Frames[0], Stretch = Stretch.None };
#endif
      }
    }

    private byte[] _photoBytes;

    public static Employee FromApplicationModel(ApplicationModel.Employee amEmployee)
    {
      var employee = new Employee();
      employee.AppEmployee = amEmployee;

      //employee.BaseFromApplicationModel(amEmployee);
      employee.LastName = amEmployee.LastName;
      employee.FirstName = amEmployee.FirstName;
      employee.TitleOfCourtesy = amEmployee.TitleOfCourtesy;
      employee.Title = amEmployee.Title;
      if (amEmployee.Photo!=null)
        employee.PhotoBytes = amEmployee.Photo;
      return employee;
    }

    public ApplicationModel.Employee ToApplicationModel()
    {
      if (AppEmployee == null)
        AppEmployee = new ApplicationModel.Employee();

      //BaseToApplicationModel(amEmployee);
      AppEmployee.LastName = LastName;
      AppEmployee.FirstName = FirstName;
      AppEmployee.Title = Title;
      AppEmployee.TitleOfCourtesy = TitleOfCourtesy;
      return AppEmployee;
    }

    /// <summary>
    /// Application model employee.
    /// </summary>
    internal ApplicationModel.Employee AppEmployee { get; set; }
  }
}

