﻿using System;
using System.ComponentModel;
using System.IO;
using System.Linq.Expressions;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Lobs.ViewModel
{

  public class Observable<T> : INotifyPropertyChanged
  {
    public Observable()
    { }

    public Observable(T value)
    {
      _value = value;
    }

    public T Value
    {
      get { return _value; }
      set
      {
        _value = value;
        if (PropertyChanged != null)
          PropertyChanged(this, new PropertyChangedEventArgs("Value"));
      }
    }

    public static implicit operator T(Observable<T> val)
    {
      return val._value;
    }

    public event PropertyChangedEventHandler PropertyChanged;

    T _value;
  }

  /// <summary>
  /// View model Product.
  /// This implementation directly uses the properties of the application model product.
  /// </summary>
  public class Product : ViewModelBase
  {
    public Product()
    {
      _appProduct = new ApplicationModel.Product();
    }

    public string Name
    {
      get { return _appProduct.Name; }
      set { if (!ReferenceEquals(_appProduct.Name, value)) { _appProduct.Name = value; RaisePropertyChanged(() => Name); } }
    }

    public ApplicationModel.Category Category
    {
      get { return _appProduct.Category; }
      set { if (!ReferenceEquals(_appProduct.Category, value)) { _appProduct.Category = value; RaisePropertyChanged(() => Category); } }
    }

    public decimal UnitPrice
    {
      get { return _appProduct.UnitPrice; }
      set { if (_appProduct.UnitPrice != value) { _appProduct.UnitPrice = value; RaisePropertyChanged(() => UnitPrice); } }
    }

    public override string ToString()
    {
      return Name;
    }

    public static Product FromApplicationModel(ApplicationModel.Product amProduct)
    {
      var product = new Product();
      product._appProduct = amProduct;
      return product;
    }

    public ApplicationModel.Product ToApplicationModel()
    {
      if (_appProduct == null)
        _appProduct = new ApplicationModel.Product();
      return _appProduct;
    }

    /// <summary>
    /// Application model product.
    /// </summary>
    ApplicationModel.Product _appProduct;
  }
}

