﻿using System;
using System.Text;
using System.Runtime.InteropServices;

namespace Frmwrk.Win32.NativeMethods
{
  // ReSharper disable InconsistentNaming

  /// <summary>
  /// Encapsulates some native Windows functions of User32.dll.
  /// See Microsoft SDK documentation for further details.
  /// </summary>
  public static class User32
  {
    /// <summary>
    /// This function determines whether a key is up or down at the time the function is called, 
    /// and whether the key was pressed after a previous call to GetAsyncKeyState.
    /// </summary>
    [DllImport("user32.dll")]
    public static extern short GetAsyncKeyState(int key);

    /// <summary>
    /// Native WindowProc delegate.
    /// </summary>
    public delegate int WindowProc(IntPtr hwnd, int msg, IntPtr wparam, IntPtr lparam);


    #region WM_*

    public const int WM_CREATE = 0x0001;
    public const int WM_DESTROY = 0x0002;
    public const int WM_MOVE = 0x0003;
    public const int WM_SIZE = 0x0005;
    public const int WM_ACTIVATE = 0x0006;
    public const int WA_INACTIVE = 0;
    public const int WA_ACTIVE = 1;
    public const int WA_CLICKACTIVE = 2;

    public const int WM_SETTEXT = 0x000C;
    public const int WM_FONTCHANGE = 0x001D;
    public const int WM_WINDOWPOSCHANGED = 0x0047;
    public const int WM_HELP = 0x53;
    public const int WM_GETICON = 0x007F;
    public const int WM_SETICON = 0x0080;
    public const int WM_NCACTIVATE = 0x0086;
    public const int WM_KEYDOWN = 0x0100;
    public const int WM_KEYUP = 0x0101;
    public const int WM_CHAR = 0x0102;
    public const int WM_DEADCHAR = 0x0103;
    public const int WM_SYSKEYDOWN = 0x0104;
    public const int WM_SYSKEYUP = 0x0105;
    public const int WM_SYSCHAR = 0x0106;
    public const int WM_SYSDEADCHAR = 0x0107;
    public const int WM_UNICHAR = 0x0109;
    public const int WM_COMMAND = 0x0111;
    public const int WM_SYSCOMMAND = 0x112;
    #endregion

    #region SC_*
    public const int SC_MaskCommand = 0xfff;
    public const int SC_MaskSource = 0xf000;
    public const int SC_CONTEXTHELP = 0x180;
    #endregion

    public const int WH_KEYBOARD = 2;
    public const int HC_ACTION = 0;

    #region WS_* & WS_EX_*
    public const uint WS_OVERLAPPED = 0x00000000;  // is zero
    public const uint WS_POPUP = 0x80000000;
    public const uint WS_CHILD = 0x40000000;
    public const uint WS_MINIMIZE = 0x20000000;
    public const uint WS_VISIBLE = 0x10000000;
    public const uint WS_DISABLED = 0x08000000;
    public const uint WS_CLIPSIBLINGS = 0x04000000;
    public const uint WS_CLIPCHILDREN = 0x02000000;
    public const uint WS_MAXIMIZE = 0x01000000;
    public const uint WS_CAPTION = 0x00C00000; // WS.BORDER | WS.DLGFRAME
    public const uint WS_BORDER = 0x00800000;
    public const uint WS_DLGFRAME = 0x00400000;
    public const uint WS_VSCROLL = 0x00200000;
    public const uint WS_HSCROLL = 0x00100000;
    public const uint WS_SYSMENU = 0x00080000;
    public const uint WS_THICKFRAME = 0x00040000;
    public const uint WS_GROUP = 0x00020000;
    public const uint WS_TABSTOP = 0x00010000;
    public const uint WS_MINIMIZEBOX = 0x00020000;
    public const uint WS_MAXIMIZEBOX = 0x00010000;

    public const uint WS_EX_DLGMODALFRAME = 0x00000001;
    public const uint WS_EX_NOPARENTNOTIFY = 0x00000004;
    public const uint WS_EX_TOPMOST = 0x00000008;
    public const uint WS_EX_ACCEPTFILES = 0x00000010;
    public const uint WS_EX_TRANSPARENT = 0x00000020;
    public const uint WS_EX_MDICHILD = 0x00000040;
    public const uint WS_EX_TOOLWINDOW = 0x00000080;
    public const uint WS_EX_WINDOWEDGE = 0x00000100;
    public const uint WS_EX_CLIENTEDGE = 0x00000200;
    public const uint WS_EX_CONTEXTHELP = 0x00000400;
    public const uint WS_EX_RIGHT = 0x00001000;
    public const uint WS_EX_LEFT = 0x00000000;
    public const uint WS_EX_RTLREADING = 0x00002000;
    public const uint WS_EX_LTRREADING = 0x00000000;
    public const uint WS_EX_LEFTSCROLLBAR = 0x00004000;
    public const uint WS_EX_RIGHTSCROLLBAR = 0x00000000;
    public const uint WS_EX_CONTROLPARENT = 0x00010000;
    public const uint WS_EX_STATICEDGE = 0x00020000;
    public const uint WS_EX_APPWINDOW = 0x00040000;
    public const uint WS_EX_OVERLAPPEDWINDOW = (WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE);
    public const uint WS_EX_PALETTEWINDOW = (WS_EX_WINDOWEDGE | WS_EX_TOOLWINDOW | WS_EX_TOPMOST);
    #endregion


    #region Button Control Styles
    public const uint BS_PUSHBUTTON = 0x00000000;
    public const uint BS_DEFPUSHBUTTON = 0x00000001;
    public const uint BS_CHECKBOX = 0x00000002;
    public const uint BS_AUTOCHECKBOX = 0x00000003;
    public const uint BS_RADIOBUTTON = 0x00000004;
    public const uint BS_3STATE = 0x00000005;
    public const uint BS_AUTO3STATE = 0x00000006;
    public const uint BS_GROUPBOX = 0x00000007;
    public const uint BS_USERBUTTON = 0x00000008;
    public const uint BS_AUTORADIOBUTTON = 0x00000009;
    public const uint BS_PUSHBOX = 0x0000000A;
    public const uint BS_OWNERDRAW = 0x0000000B;
    public const uint BS_TYPEMASK = 0x0000000F;
    public const uint BS_LEFTTEXT = 0x00000020;
    public const uint BS_TEXT = 0x00000000;
    public const uint BS_ICON = 0x00000040;
    public const uint BS_BITMAP = 0x00000080;
    public const uint BS_LEFT = 0x00000100;
    public const uint BS_RIGHT = 0x00000200;
    public const uint BS_CENTER = 0x00000300;
    public const uint BS_TOP = 0x00000400;
    public const uint BS_BOTTOM = 0x00000800;
    public const uint BS_VCENTER = 0x00000C00;
    public const uint BS_PUSHLIKE = 0x00001000;
    public const uint BS_MULTILINE = 0x00002000;
    public const uint BS_NOTIFY = 0x00004000;
    public const uint BS_FLAT = 0x00008000;
    public const uint BS_RIGHTBUTTON = BS_LEFTTEXT;
    #endregion


    #region HWND_*
    public static readonly IntPtr HWND_TOP = (IntPtr)0;
    public static readonly IntPtr HWND_BOTTOM = (IntPtr)1;
    public static readonly IntPtr HWND_TOPMOST = (IntPtr)(-1);
    public static readonly IntPtr HWND_NOTOPMOST = (IntPtr)(-2);
    public static readonly IntPtr HWND_BROADCAST = (IntPtr)0xffff;
    #endregion

    #region GWL_*
    public const int GWL_WNDPROC = -4;
    public const int GWL_HINSTANCE = -6;
    public const int GWL_HWNDPARENT = -8;
    public const int GWL_STYLE = -16;
    public const int GWL_EXSTYLE = -20;
    public const int GWL_USERDATA = -21;
    public const int GWL_ID = -12;
    #endregion

    #region SWP_*
    public const uint SWP_NOSIZE = 0x0001;
    public const uint SWP_NOMOVE = 0x0002;
    public const uint SWP_NOZORDER = 0x0004;
    public const uint SWP_NOREDRAW = 0x0008;
    public const uint SWP_NOACTIVATE = 0x0010;
    public const uint FRAMECHANGED = 0x0020;  /* The frame changedSWP_: send WM_NCCALCSIZE */
    public const uint SWP_SHOWWINDOW = 0x0040;
    public const uint SWP_HIDEWINDOW = 0x0080;
    public const uint SWP_NOCOPYBITS = 0x0100;
    public const uint NOOWNERZORDER = 0x0200;  /* Don't do SWP_owner Z ordering */
    public const uint NOSENDCHANGING = 0x0400;  /* Don'SWP_t send WM_WINDOWPOSCHANGING */
    public const uint SWP_DRAWFRAME = FRAMECHANGED;
    public const uint SWP_NOREPOSITION = NOOWNERZORDER;
    public const uint SWP_DEFERERASE = 0x2000;
    public const uint SWP_ASYNCWINDOWPOS = 0x4000;
    #endregion

    #region RDW_
    public const int RDW_INVALIDATE = 0x0001;
    public const int RDW_INTERNALPAINT = 0x0002;
    public const int RDW_ERASE = 0x0004;

    public const int RDW_VALIDATE = 0x0008;
    public const int RDW_NOINTERNALPAINT = 0x0010;
    public const int RDW_NOERASE = 0x0020;

    public const int RDW_NOCHILDREN = 0x0040;
    public const int RDW_ALLCHILDREN = 0x0080;

    public const int RDW_UPDATENOW = 0x0100;
    public const int RDW_ERASENOW = 0x0200;

    public const int RDW_FRAME = 0x0400;
    public const int RDW_NOFRAME = 0x0800;
    #endregion

    [StructLayout(LayoutKind.Sequential)]
    public struct RECT
    {
      public int left;
      public int top;
      public int right;
      public int bottom;
    }

    public delegate int HookProc(int nCode, IntPtr wParam, IntPtr lParam);

    [DllImport("user32.dll")]
    public static extern IntPtr GetActiveWindow();

    [DllImport("user32.dll")]
    public static extern IntPtr SetForegroundWindow(IntPtr hwnd);

    [DllImport("user32.dll")]
    public static extern int GetWindowText(IntPtr hwnd, StringBuilder text, int size);

    [DllImport("user32.dll")]
    public static extern bool EnableWindow(IntPtr hwnd, bool fEnable);

    [DllImport("user32.dll")]
    public static extern IntPtr SetParent(IntPtr hwnd, IntPtr hwndNewParent);

    [DllImport("user32.dll")]
    public static extern int GetWindowLong(IntPtr hwnd, int index);

    [DllImport("user32.dll")]
    public static extern int SetWindowLong(IntPtr hwnd, int index, int val);

    [DllImport("user32.dll", EntryPoint = "SetWindowLong")]
    [Obsolete("Use SubclassWindow")]
    public static extern int SetWindowLong2(IntPtr hwnd, int index, WindowProc wndProc);

    [DllImport("user32.dll")]
    public static extern IntPtr SetWindowsHookEx(int idHook, HookProc hook, IntPtr hmod, int threadId);

    [DllImport("user32.dll")]
    public static extern IntPtr UnhookWindowsHookEx(IntPtr hook);

    /// <summary>
    /// Subclasses a window to a new window procedure.
    /// </summary>
    public static IntPtr SubclassWindow(IntPtr hwnd, WindowProc wndProc)
    {
      return SetWindowProc(hwnd, GWL_WNDPROC, wndProc);
    }
    /// <summary>
    /// EntryPoint = "SetWindowLongPtr" / "SetWindowLongPtrA" / "SetWindowLongPtrW" do not work!?
    /// </summary>
    [DllImport("user32.dll", EntryPoint = "SetWindowLong")]
    private static extern IntPtr SetWindowProc(IntPtr hwnd, int index, WindowProc wndProc);

    [DllImport("user32.dll")]
    public static extern int SendMessage(IntPtr hwnd, uint message, IntPtr wParam, IntPtr lParam);

    [DllImport("user32.dll")]
    public static extern int PostMessage(IntPtr hwnd, uint message, IntPtr wParam, IntPtr lParam);

    [DllImport("user32.dll", EntryPoint = "CallWindowProcW")]
    public static extern int CallWindowProc(IntPtr wndProc, IntPtr hwnd, int message,
      IntPtr wParam, IntPtr lParam);

    [DllImport("user32.dll")]
    public static extern int RedrawWindow(IntPtr hwnd, IntPtr lprcUpdate, IntPtr hrgnUpdate, uint flags);

    [DllImport("user32.dll", EntryPoint = "DefWindowProcW", CharSet = CharSet.Unicode)]
    public static extern int DefWindowProc(IntPtr hwnd, int message, IntPtr wParam, IntPtr lParam);

    [DllImport("user32.dll")]
    public static extern int CallNextHookEx(IntPtr hhk, int nCode, IntPtr wParam, IntPtr lParam);

    [DllImport("user32.dll")]
    public static extern IntPtr GetSystemMenu(IntPtr hwnd, bool revert);

    [DllImport("user32.dll")]
    public static extern int GetMenuItemCount(IntPtr hmenu);

    [DllImport("user32.dll")]
    public static extern bool DeleteMenu(IntPtr hmenu, int position, int flags);

    [DllImport("user32.dll")]
    public static extern bool EnableMenuItem(IntPtr hmenu, int position, int flags);

    [DllImport("user32.dll")]
    public static extern IntPtr LoadIcon(IntPtr hinst, IntPtr lpIconName);

    [DllImport("user32.dll")]
    public static extern bool SetWindowPos(IntPtr hwnd, IntPtr hwndInsertAfter,
      int x, int y, int cx, int cy, uint swp);

    [DllImport("user32.dll")]
    public static extern bool GetWindowRect(IntPtr hwnd, ref RECT rect);

    [DllImport("user32.dll")]
    public static extern bool GetClientRect(IntPtr hwnd, ref RECT rect);

    [DllImport("user32.dll")]
    public static extern IntPtr FindWindowEx(IntPtr hwndParent, IntPtr hwndChildAfter, string lpszClass, string lpszWindow);

    public struct MessageBeepType
    {
      public const uint Simple = 0xFFFFFFFF;
      public const uint OK = 0x00000000;
      public const uint Hand = 0x00000010;
      public const uint Question = 0x00000020;
      public const uint Exclamation = 0x00000030;
      public const uint Asterisk = 0x00000040;
    }
    [DllImport("user32.dll")]
    public static extern bool MessageBeep(uint type);

    /* WinHelp */
    [DllImport("user32.dll", EntryPoint = "WinHelpW", CharSet = CharSet.Unicode)]
    public static extern int WinHelp(IntPtr hwndMain,
      string lpszHelp,
      uint uCommand,
      string dwData);
    /*
    WINUSERAPI BOOL WINAPI WinHelpW(
    __in_opt HWND hWndMain,
    __in_opt LPCWSTR lpszHelp,
    __in UINT uCommand,
    __in ULONG_PTR dwData);
     */

    /*
     * Commands to pass to WinHelp()
     */
    public struct WinHelpCommands
    {
      public const uint HELP_CONTEXT = 0x0001; /* Display topic in ulTopic */
      public const uint HELP_QUIT = 0x0002;  /* Terminate help */
      public const uint HELP_INDEX = 0x0003;  /* Display index */
      public const uint HELP_CONTENTS = 0x0003;
      public const uint HELP_HELPONHELP = 0x0004;  /* Display help on using help */
      public const uint HELP_SETINDEX = 0x0005;  /* Set current Index for multi index help */
      public const uint HELP_SETCONTENTS = 0x0005;
      public const uint HELP_CONTEXTPOPUP = 0x0008;
      public const uint HELP_FORCEFILE = 0x0009;
      public const uint HELP_KEY = 0x0101;  /* Display topic for keyword in offabData */
      public const uint HELP_COMMAND = 0x0102;
      public const uint HELP_PARTIALKEY = 0x0105;
      public const uint HELP_MULTIKEY = 0x0201;
      public const uint HELP_SETWINPOS = 0x0203;
      //#if(WINVER >= 0x0400)
      public const uint HELP_CONTEXTMENU = 0x000a;
      public const uint HELP_FINDER = 0x000b;
      public const uint HELP_WM_HELP = 0x000c;
      public const uint HELP_SETPOPUP_POS = 0x000d;

      public const uint HELP_TCARD = 0x8000;
      public const uint HELP_TCARD_DATA = 0x0010;
      public const uint HELP_TCARD_OTHER_CALLER = 0x0011;

      // These are in winhelp.h in Win95.
      public const uint IDH_NO_HELP = 28440;
      public const uint IDH_MISSING_CONTEXT = 28441; // Control doesn't have matching help context
      public const uint IDH_GENERIC_HELP_BUTTON = 28442; // Property sheet help button
      public const uint IDH_OK = 28443;
      public const uint IDH_CANCEL = 28444;
      public const uint IDH_HELP = 28445;
      //#endif /* WINVER >= 0x0400 */
    }

    const uint HH_DISPLAY_TOPIC = 0x0000;
    const uint HH_KEYWORD_LOOKUP = 0x000D;

    public static IntPtr HtmlHelp(IntPtr hWndCaller, string helpFile, /*UInt32 command,*/ string data)
    {
      IntPtr ip = HtmlHelp(hWndCaller, helpFile, HH_DISPLAY_TOPIC, null);
      if (!string.IsNullOrEmpty(data))
      {
        HH_AKLINK aklink = new HH_AKLINK();
        aklink.pszKeywords = data;
        aklink.fIndexOnFail = true;
        return HtmlHelp(hWndCaller, helpFile, HH_KEYWORD_LOOKUP, aklink);
      }
      return ip;
    }

    [DllImport("hhctrl.ocx", /*EntryPoint = "HtmlHelpW",*/ CharSet = CharSet.Auto, SetLastError = true)]
    static extern IntPtr HtmlHelp(IntPtr hWndCaller, string helpFile, UInt32 command, HH_AKLINK data);

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
    public class HH_AKLINK
    {
      internal int cbStruct = Marshal.SizeOf(typeof(HH_AKLINK));
      internal bool fReserved;
      internal string pszKeywords;
      internal string pszUrl;
      internal string pszMsgText;
      internal string pszMsgTitle;
      internal string pszWindow;
      internal bool fIndexOnFail;
    }
  }
}
