﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Interop;
#if !SILVERLIGHT
using Frmwrk.Win32.NativeMethods;
#endif

namespace Frmwrk.Windows.Basis
{
  /// <summary>
  /// Base class for all dialog windows.
  /// </summary>
  /// <remarks>
  /// Just for demonstration what to do in such a base class.
  /// </remarks>
  public class DialogBase : WindowBase
  {
    protected DialogBase()
    {
#if !SILVERLIGHT
      WindowStartupLocation = WindowStartupLocation.CenterOwner;
      ShowInTaskbar = false;
#endif
    }

#if !SILVERLIGHT
    /// <summary>
    /// Removes unused items from the system menu of a dialog window.
    /// </summary>
    protected void MakeUpDialogSysMenu(bool sizeable, bool closeable)
    {
      UITools.MakeUpDialogSysMenu(this, sizeable, closeable);
    }

    /// <summary>
    /// Removes unused items from the system menu of a dialog window.
    /// </summary>
    protected void MakeUpDialogSysMenu(bool sizeable)
    {
      UITools.MakeUpDialogSysMenu(this, sizeable);
    }

    protected void MakeUpDialogSysMenu()
    {
      UITools.MakeUpDialogSysMenu(this,
        ResizeMode == ResizeMode.CanResizeWithGrip ||
        ResizeMode == ResizeMode.CanResize);
    }

    protected override void OnClosed(EventArgs e)
    {
      HelpButton = false; // We must free our HwndSource.
      base.OnClosed(e);
    }

    /// <summary>
    /// Sets a value indicating whether the window has a help button in the title bar.
    /// </summary>
    protected bool HelpButton
    {
      set { SetHelpHandler(value); UITools.ShowHelpButton(this, value); }
    }

    private void SetHelpHandler(bool value)
    {
      if (value)
      {
        // Subclass the Window
        if (hookHelpHandler == null)
        {
          WindowInteropHelper wih = new WindowInteropHelper(this);
          hookHelpHandler = new HwndSourceHook(HookHelpHandler);
          hwndSource = HwndSource.FromHwnd(wih.Handle);
          Debug.Assert(hwndSource != null);
          hwndSource.AddHook(hookHelpHandler);
        }
      }
      else
      {
        // Remove the subclass???
        if (hookHelpHandler != null)
        {
          hwndSource.RemoveHook(hookHelpHandler);
          hwndSource.Dispose();
          hwndSource = null;
          hookHelpHandler = null;
        }
      }
    }

    public IntPtr HookHelpHandler(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
    {
      switch (msg)
      {
        case User32.WM_HELP:
          handled = true;
          OnHelpButtonClicked();
          return (IntPtr)1;

        case User32.WM_SYSCOMMAND:
          if (((int)wParam & User32.SC_MaskCommand) == User32.SC_CONTEXTHELP)
          {
            handled = true;
            OnHelpButtonClicked();
            return (IntPtr)1;
          }
          break;
      }
      return (IntPtr)0;
    }


    protected virtual void OnHelpButtonClicked()
    {
      EventHandler handler = HelpButtonClicked;
      if (handler != null)
      {
        // Invokes the delegates. 
        EventArgs e = new EventArgs();
        handler(this, e);
      }
    }

    /// <summary>
    /// Occurs when the help button on the title bar is clicked.
    /// </summary>
    [Description("Occurs when the help button on the title bar is clicked."), Category("Preview Properties")]
    public event EventHandler HelpButtonClicked;

    /// <summary>
    /// Removes the icon from the system menu box.
    /// </summary>
    protected void RemoveIcon()
    {
      UITools.RemoveIcon(this);
    }

    /// <summary>
    /// Patches the frame window to a non-sizable standard dialg window with no icon, no minimize box, no maximize box, 
    /// no icon in task bar, and fewer items in the system menu.
    /// </summary>
    protected void MakeStandardDialog()
    {
      ResizeMode = ResizeMode.NoResize;
      ShowInTaskbar = false;
      UITools.MakeUpDialogSysMenu(this, false);
      UITools.RemoveIcon(this);
    }

    // Not needed if we hook WM_HELP:
    //protected override void OnPreviewKeyDown(KeyEventArgs e)
    //{
    //  if (e.Key == Key.F1)
    //  {
    //    if (!string.IsNullOrEmpty(this.HelpContext))
    //    {
    //      e.Handled = true;
    //      WinHelp.ShowHelp(this, this.HelpContext);
    //      return;
    //    }
    //  }
    //  base.OnPreviewKeyDown(e);
    //}

    /// <summary>
    /// Patches the frame window to a sizable standard dialg window with no icon, no minimize box, no maximize box, 
    /// no icon in task bar, and fewer items in the system menu.
    /// </summary>
    protected void MakeSizableStandardDialog()
    {
      ResizeMode = ResizeMode.CanResizeWithGrip;
      ShowInTaskbar = false;
      UITools.MakeUpDialogSysMenu(this, true);
      UITools.RemoveIcon(this);
    }

    HwndSource hwndSource;
    HwndSourceHook hookHelpHandler;
#endif
  }
}
