﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Windows;

namespace Frmwrk.Windows.ViewModel
{
  /// <summary>
  /// Base class for all view model classes.
  /// </summary>
  public class ViewModelBase : INotifyPropertyChanged
  {
    protected void EnsureMainThread()
    {
#if SILVERLIGHT
      if (!Deployment.Current.CheckAccess())
        throw new InvalidOperationException("Must be used in main thread.");
#endif
    }

    /// <summary>
    /// Raises the property changed event for the specified property expression if PropertyChanged event is defined.
    /// </summary>
    protected void RaisePropertyChanged<TResult>(Expression<Func<TResult>> propertyExpression)
    {
      RaisePropertyChanged(((MemberExpression)propertyExpression.Body).Member.Name);
    }

    /// <summary>
    /// Raises the property changed event for the specified property name if PropertyChanged event is defined.
    /// </summary>
    protected void RaisePropertyChanged(string propertyName)
    {
      if (PropertyChanged != null)
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    }

    /// <summary>
    /// Property changed event for interface INotifyPropertyChanged.
    /// </summary>
    public event PropertyChangedEventHandler PropertyChanged;
  }

#if true_
  public static class PropertyChangedEventHandlerExtensions
  {
    /// <summary>
    /// Raises the specified property changed event handler if it is defined.
    /// </summary>
    public static void Raise(this PropertyChangedEventHandler handler, object sender, Expression<Func<object>> propertyExpression)
    {
      if (handler != null)
        handler(sender, new PropertyChangedEventArgs((((MemberExpression)propertyExpression.Body).Member.Name)));
    }

    /// <summary>
    /// Raises the specified property changed event handler if it is defined.
    /// </summary>
    public static void Raise(this PropertyChangedEventHandler handler, object sender, Expression<Func<ValueType>> propertyExpression)
    {
      if (handler != null)
        handler(sender, new PropertyChangedEventArgs((((MemberExpression)((UnaryExpression)propertyExpression.Body).Operand).Member.Name)));
    }

    /// <summary>
    /// Raises the specified property changed event handler if it is defined.
    /// </summary>
    public static void Raise<T, TResult>(this PropertyChangedEventHandler handler, T sender, Expression<Func<T, TResult>> propertyExpression)
    {
      if (handler != null)
        handler(sender, new PropertyChangedEventArgs(((MemberExpression)propertyExpression.Body).Member.Name));
    }
  }
#endif
}
