﻿using System;
using System.Diagnostics;
using System.Text;
using System.Windows;
using Frmwrk.Win32;
using Frmwrk.Win32.NativeMethods;
using System.Windows.Interop;

namespace Frmwrk.Windows
{
  /// <summary>
  /// Useful collection of functions to encapsulate some Windows hacks.
  /// </summary>
  public static class UITools
  {
    /// <summary>
    /// Removes unused items from the system menu of a dialog window. In a dialog
    /// windows (that is not sizeable) the items Resore, Move, Size, Maximize and
    /// the separator have to be removed.
    /// </summary>
    public static void MakeUpDialogSysMenu(Window window, bool sizeable)
    {
      if (window == null)
        throw new ArgumentNullException("window");

      //const int MF_BYCOMMAND  = 0x00000000;
      const int MF_BYPOSITION = 0x00000400;

      IntPtr hwnd = new WindowInteropHelper(window).Handle;
      if (hwnd != IntPtr.Zero)
      {
        uint ws = (uint)User32.GetWindowLong(hwnd, User32.GWL_STYLE);
        ws &= ~(User32.WS_MAXIMIZEBOX | User32.WS_MINIMIZEBOX);
        User32.SetWindowLong(hwnd, User32.GWL_STYLE, (int)ws);

        uint wsex = (uint)User32.GetWindowLong(hwnd, User32.GWL_EXSTYLE);
        wsex |= User32.WS_EX_DLGMODALFRAME;
        User32.SetWindowLong(hwnd, User32.GWL_EXSTYLE, (int)wsex);

        User32.SendMessage(hwnd, User32.WM_SETICON, IntPtr.Zero, IntPtr.Zero);
        User32.SendMessage(hwnd, User32.WM_SETICON, new IntPtr(1), IntPtr.Zero);
        User32.RedrawWindow(hwnd, IntPtr.Zero, IntPtr.Zero, 0x401);

        //wsex &= ~User32.WS_EX_DLGMODALFRAME;
        //User32.SetWindowLong(hwnd, User32.GWL_EXSTYLE, (int)wsex);

        IntPtr hmenu = User32.GetSystemMenu(hwnd, false);
        if (hmenu != IntPtr.Zero)
        {
          //DaSt TODO: Wäre es möglich wenn anfänglich z.B. 20 Einträge vorhanden sind
          //und beim zweiten Aufruf immer noch mehr als 7 Einträge existiert?
          Debug.Assert(User32.GetMenuItemCount(hmenu) >= 7, "Must not call MakeUpDialogSysMenu twice.");

          // 4STLA: TODO: nur Einträge löschen, die disabled sind!!!
          // Maximize darf nicht immer gelöscht werden, wenn ein Dialog sizeable ist!!!
          User32.DeleteMenu(hmenu, 5, MF_BYPOSITION);
          User32.DeleteMenu(hmenu, 4, MF_BYPOSITION);
          User32.DeleteMenu(hmenu, 3, MF_BYPOSITION);
          if (!sizeable)
            User32.DeleteMenu(hmenu, 2, MF_BYPOSITION);
          User32.DeleteMenu(hmenu, 0, MF_BYPOSITION);
        }
      }
    }

    /// <summary>
    /// Removes unused items from the system menu of a dialog window. In a dialog
    /// windows (that is not sizeable) the items Resore, Move, Size, Maximize and
    /// the separator have to be removed.
    /// </summary>
    public static void MakeUpDialogSysMenu(Window window, bool sizeable, bool closeable)
    {
      if (window == null)
        throw new ArgumentNullException("window");

      //const int MF_BYCOMMAND  = 0x00000000;
      const int MF_BYPOSITION = 0x00000400;
      //const int MF_ENABLED    = 0x00000000;
      const int MF_GRAYED = 0x00000001;
      //const int MF_DISABLED   = 0x00000002;

      IntPtr hmenu = User32.GetSystemMenu(new WindowInteropHelper(window).Handle, false);
      if (hmenu != IntPtr.Zero)
      {
        Debug.Assert(User32.GetMenuItemCount(hmenu) >= 7, "Must not call MakeUpDialogSysMenu twice.");

        if (!closeable)
          User32.EnableMenuItem(hmenu, 6, MF_BYPOSITION | MF_GRAYED);

        // Maximize darf nicht immer gelöscht werden, wenn ein Dialog sizeable ist!!!
        User32.DeleteMenu(hmenu, 5, MF_BYPOSITION);
        User32.DeleteMenu(hmenu, 4, MF_BYPOSITION);
        User32.DeleteMenu(hmenu, 3, MF_BYPOSITION);
        if (!sizeable)
          User32.DeleteMenu(hmenu, 2, MF_BYPOSITION);
        User32.DeleteMenu(hmenu, 0, MF_BYPOSITION);
      }
    }

    public static void MakeUpDialogSysMenu(Window window)
    {
      MakeUpDialogSysMenu(window, false);
    }

    /// <summary>
    /// Sets a value indicating whether the window has a help button in the title bar.
    /// </summary>
    public static void ShowHelpButton(Window window, bool value)
    {
      if (window == null)
        throw new ArgumentNullException("window");

      IntPtr hwnd = new WindowInteropHelper(window).Handle;
      uint ws = (uint)User32.GetWindowLong(hwnd, User32.GWL_EXSTYLE);
      if (value)
        ws |= User32.WS_EX_CONTEXTHELP;
      else
        ws &= ~User32.WS_EX_CONTEXTHELP;
      User32.SetWindowLong(hwnd, User32.GWL_EXSTYLE, (int)ws);
    }

    /// <summary>
    /// Removes the icon from the window's system menu box.
    /// </summary>
    public static void RemoveIcon(Window window)
    {
      const int ICON_SMALL = 0;
      const int ICON_BIG = 1;

      IntPtr hwnd = new WindowInteropHelper(window).Handle;
      if (hwnd != IntPtr.Zero)
      {
        // Does not work anymore since Vista
        User32.SendMessage(hwnd, User32.WM_SETICON, (IntPtr)ICON_SMALL, IntPtr.Zero);
        User32.SendMessage(hwnd, User32.WM_SETICON, (IntPtr)ICON_BIG, IntPtr.Zero);
        User32.RedrawWindow(hwnd, IntPtr.Zero, IntPtr.Zero, User32.RDW_INVALIDATE | User32.RDW_UPDATENOW | User32.RDW_FRAME);
      }
    }
  }
}
