﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Frmwrk.ComponentModel;
using Lobs.ServiceReference2;

namespace Lobs.DataModel
{
  public class WcfDataProvider : IDataProvider
  {
    public WcfDataProvider(string parameter)
    { }

    public static WcfDataProvider CreateDataProvider()
    {
      return new WcfDataProvider(null);
    }

    public IEnumerable<Category> LoadCategories()
    {
      var client = new Service2Client();
      var syncHelper = new SyncCallHelper<LoadCategoriesCompletedEventArgs>();
      client.LoadCategoriesCompleted+= syncHelper.OnCompleted;
      client.LoadCategoriesAsync(syncHelper);
      syncHelper.WaitUntilSuccesOrFail();
      return syncHelper.EventArgs.Result;
    }

    public IEnumerable<Shipper> LoadShippers()
    {
      var client = new Service2Client();
      var syncHelper = new SyncCallHelper<LoadShippersCompletedEventArgs>();
      client.LoadShippersCompleted += syncHelper.OnCompleted;
      client.LoadShippersAsync(syncHelper);
      syncHelper.WaitUntilSuccesOrFail();
      return syncHelper.EventArgs.Result;
    }

    public IEnumerable<Employee> LoadEmployees()
    {
      var client = new Service2Client();
      var syncHelper = new SyncCallHelper<LoadEmployeesCompletedEventArgs>();
      client.LoadEmployeesCompleted += syncHelper.OnCompleted;
      client.LoadEmployeesAsync(syncHelper);
      syncHelper.WaitUntilSuccesOrFail();
      return syncHelper.EventArgs.Result;
    }

    public byte[] LoadEmployeePhoto(string id)
    {
      var client = new Service2Client();
      var syncHelper = new SyncCallHelper<LoadEmployeePhotoCompletedEventArgs>();
      client.LoadEmployeePhotoCompleted+= syncHelper.OnCompleted;
      client.LoadEmployeePhotoAsync(id, syncHelper);
      syncHelper.WaitUntilSuccesOrFail();
      return syncHelper.EventArgs.Result;
    }

    public void SaveEmployee(Employee employee)
    {
      var client = new Service2Client();
      var syncHelper = new SyncCallHelper<AsyncCompletedEventArgs>();
      client.SaveEmployeeCompleted += syncHelper.OnCompleted;
      client.SaveEmployeeAsync(employee, syncHelper);
      syncHelper.WaitUntilSuccesOrFail();
    }

    public void DeleteEmployee(Employee employee)
    {
      var client = new Service2Client();
      var syncHelper = new SyncCallHelper<AsyncCompletedEventArgs>();
      client.DeleteEmployeeCompleted += syncHelper.OnCompleted;
      client.DeleteEmployeeAsync(employee, syncHelper);
      syncHelper.WaitUntilSuccesOrFail();
    }

    public int SaveEmployees(IEnumerable<Employee> employees)
    {
      var list = new List<Employee>();
      list.AddRange(employees);

      var client = new Service2Client();
      var syncHelper = new SyncCallHelper<SaveEmployeesCompletedEventArgs>();
      client.SaveEmployeesCompleted += syncHelper.OnCompleted;
      client.SaveEmployeesAsync(list, syncHelper);
      syncHelper.WaitUntilSuccesOrFail();
      return syncHelper.EventArgs.Result;
    }

    public IEnumerable<Product> LoadProducts()
    {
      var client = new Service2Client();
      var syncHelper = new SyncCallHelper<LoadProductsCompletedEventArgs>();
      client.LoadProductsCompleted += syncHelper.OnCompleted;
      client.LoadProductsAsync(syncHelper);
      syncHelper.WaitUntilSuccesOrFail();
      return syncHelper.EventArgs.Result;
    }

    public void SaveProduct(Product product)
    {
      throw new NotImplementedException();
    }

  }
}
